#include "fmcadc100m14b4cha.h"

static void adc100m14b4cha_lib_set_up(struct m_suite *m_suite)
{
	struct adc_dev *adc;
	int err;

	err = adc_init();
	m_assert_int_eq(0, err);

	adc = adc_open(ADC100M14B4CHA_TYPE_NAME, adc_devid, 0, 0, ADC_F_FLUSH);
	m_assert_mem_not_null(adc);

	m_suite->private = adc;

	srandom(time(NULL));
}
static void adc100m14b4cha_lib_tear_down(struct m_suite *m_suite)
{
	struct adc_dev *adc = m_suite->private;

	if (!adc)
		return;

	adc_close(adc);
	adc_exit();
}


static inline void adc100m14b4cha_lib_cfg_test_readback(struct adc_dev *adc,
							struct adc_conf *cfg,
							unsigned int index,
							uint32_t value)
{
	uint32_t value_rb;
	int err;

	/* try random configuration */
	adc_set_conf(cfg, index, value);
	err = adc_apply_config(adc, 0 , cfg);
	m_assert_int_eq(0, err);
	err = adc_retrieve_config(adc, cfg);
	m_assert_int_eq(0, err);
	err = adc_get_conf(cfg, index, &value_rb);
	m_assert_int_eq(0, err);
	m_assert_int_eq(value, value_rb);
}


#define adc100m14b4cha_lib_cfg_test1_desc "It checks the pre-sample read/write"
static void adc100m14b4cha_lib_cfg_test1(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;

	/* test special values first */
	adc_set_conf(&cfg, ADC_CONF_ACQ_PRE_SAMP, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);

	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	m_assert_int_eq(0, cfg.value[ADC_CONF_ACQ_PRE_SAMP]);

	/* try random configuration */
	value = random() & 0xFFF;
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_ACQ_PRE_SAMP, value);
}

#define adc100m14b4cha_lib_cfg_test2_desc "It checks the post-sample read/write"
static void adc100m14b4cha_lib_cfg_test2(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value, value_rb;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;

	/* test special values */
	adc_set_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EINVAL, errno);

	adc_set_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EINVAL, errno);

	adc_set_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, 2);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	err = adc_get_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, &value_rb);
	m_assert_int_eq(0, err);
	m_assert_int_eq(2, value_rb);

	/* try random configuration */
	value = (random() & 0xFFF) + 2; /* skip invalid values 0 and 1 */
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_ACQ_POST_SAMP, value);
}

#define adc100m14b4cha_lib_cfg_test3_desc "It checks the under-sample read/write"
static void adc100m14b4cha_lib_cfg_test3(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;

	/* test special values */
	adc_set_conf(&cfg, ADC_CONF_ACQ_UNDERSAMPLE, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	m_assert_int_eq(1, cfg.value[ADC_CONF_ACQ_UNDERSAMPLE]);

	adc_set_conf(&cfg, ADC_CONF_ACQ_UNDERSAMPLE, 1);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(0, err);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	m_assert_int_eq(1, cfg.value[ADC_CONF_ACQ_UNDERSAMPLE]);

	/* try random number */
	value = (random() & 0xFF) + 1; /* skip special value 0 */
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_ACQ_UNDERSAMPLE, value);
}

#define adc100m14b4cha_lib_cfg_test4_desc "It checks the frequency read/write"
static void adc100m14b4cha_lib_cfg_test4(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;
	adc_set_conf_mask(&cfg, ADC_CONF_ACQ_FREQ_HZ);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	m_assert_int_range(100000000 - 5000, 100000000 + 5000,
			   cfg.value[ADC_CONF_ACQ_FREQ_HZ]);

	/* we should fail to write, this is read-only */
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EACCES, errno);
}

#define adc100m14b4cha_lib_cfg_test5_desc "It checks the resolution-bits read/write"
static void adc100m14b4cha_lib_cfg_test5(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;
	adc_set_conf_mask(&cfg, ADC_CONF_ACQ_N_BITS);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);
	m_assert_int_eq(14, cfg.value[ADC_CONF_ACQ_N_BITS]);

	/* we should fail to write, this is read-only */
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EACCES, errno);
}

#define adc100m14b4cha_lib_cfg_test6_desc "It checks the n-shot read/write"
static void adc100m14b4cha_lib_cfg_test6(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int err;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;

	/* test special values */
	adc_set_conf(&cfg, ADC_CONF_ACQ_N_SHOTS, 0);
	err = adc_apply_config(adc, 0 , &cfg);
	m_assert_int_eq(-1, err);
	m_assert_int_eq(EINVAL, errno);

	/* try random number */
	value = (random() & 0xF) + 1; /* +1 to avoid the special case 0 */
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					       ADC_CONF_ACQ_N_SHOTS, value);
}

#define adc100m14b4cha_lib_cfg_test7_desc "It checks the channel voltage range read/write"
static void adc100m14b4cha_lib_cfg_test7(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value[] = {0x23, 0x11, 0x45, 0x00};
	int err, i, k;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_CHN;

	/* loop over the 4 channels */
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		/* loop over the 4 possible ranges */
		for (k = 0; k < 4; ++k)
			adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
							     ADC_CONF_CHN_RANGE, value[k]);
		/* try invalid config */
		adc_set_conf(&cfg, ADC_CONF_CHN_RANGE, 0xFF);
		err = adc_apply_config(adc, 0 , &cfg);
		m_assert_int_eq(-1, err);
		m_assert_int_eq(EINVAL, errno);
	}
}


#define adc100m14b4cha_lib_cfg_test8_desc "It checks the channel 50 Ohm termination read/write"
static void adc100m14b4cha_lib_cfg_test8(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value[] = {3, 2, 1 ,0};
	int err, i, k;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_CHN;

	/* loop over the 4 channels */
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		/* 0 will be 0, other values are translated to 1 */
		for (k = 0; k < 4; ++k) {
			adc_set_conf(&cfg, ADC_CONF_CHN_TERMINATION, value[k]);
			err = adc_apply_config(adc, 0 , &cfg);
			m_assert_int_eq(0, err);
			err = adc_retrieve_config(adc, &cfg);
			m_assert_int_eq(0, err);
			m_assert_int_eq(!!value[k], cfg.value[ADC_CONF_CHN_TERMINATION]);
		}
	}
}

#define adc100m14b4cha_lib_cfg_test9_desc "It checks the channel offset read/write"
static void adc100m14b4cha_lib_cfg_test9(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int err, i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_CHN;

	/* loop over the 4 channels */
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		/* valid range [-5000, 5000] so 0x1FFF */
		value = random() & 0xFFF; /* positive range */
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_CHN_OFFSET, value);

		value = ~(random() & 0xFFF) + 1; /* negative range */
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_CHN_OFFSET, value);

		/* try invalid config */
		value = random() << 14;
		adc_set_conf(&cfg, ADC_CONF_CHN_OFFSET, value);
		err = adc_apply_config(adc, 0 , &cfg);
		m_assert_int_eq(-1, err);
		m_assert_int_eq(EINVAL, errno);
	}
}

#define adc100m14b4cha_lib_cfg_test10_desc "It checks the channel saturation read/write"
static void adc100m14b4cha_lib_cfg_test10(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int err, i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_CHN;

	/* loop over the 4 channels */
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		value = random() & 0x7FFF; /* max valid value 0x7FFF */
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_CHN_SATURATION, value);

		/* try invalid config */
		value = random() << 14;
		adc_set_conf(&cfg, ADC_CONF_CHN_SATURATION, value);
		err = adc_apply_config(adc, 0 , &cfg);
		m_assert_int_eq(-1, err);
		m_assert_int_eq(EINVAL, errno);
	}
}

#define adc100m14b4cha_lib_cfg_test11_desc "It checks the external trigger enable read/write"
static void adc100m14b4cha_lib_cfg_test11(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;

	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_TRG_EXT_ENABLE, 1);
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_TRG_EXT_ENABLE, 0);
}

#define adc100m14b4cha_lib_cfg_test12_desc "It checks the external trigger polarity read/write"
static void adc100m14b4cha_lib_cfg_test12(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;

	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_TRG_EXT_POLARITY, 1);
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_TRG_EXT_POLARITY, 0);
}

#define adc100m14b4cha_lib_cfg_test13_desc "It checks the external trigger delay read/write"
static void adc100m14b4cha_lib_cfg_test13(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	value = random();
	adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
					     ADC_CONF_TRG_EXT_DELAY, value);
}

#define adc100m14b4cha_lib_cfg_test14_desc "It checks the threshold trigger enable read/write"
static void adc100m14b4cha_lib_cfg_test14(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		cfg.flags = 0;
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_ENABLE, 1);
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_ENABLE, 0);
	}
}

#define adc100m14b4cha_lib_cfg_test15_desc "It checks the threshold trigger polarity read/write"
static void adc100m14b4cha_lib_cfg_test15(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	int i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		cfg.flags = 0;
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_POLARITY, 1);
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_POLARITY, 0);
	}
}

#define adc100m14b4cha_lib_cfg_test16_desc "It checks the threshold trigger delay read/write"
static void adc100m14b4cha_lib_cfg_test16(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		cfg.flags = 0;
		value = random();
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_DELAY, value);
	}
}

#define adc100m14b4cha_lib_cfg_test17_desc "It checks the threshold trigger threshold read/write"
static void adc100m14b4cha_lib_cfg_test17(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		cfg.flags = 0;
		value = random() & 0xFFFF;
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_THRESHOLD, value);
	}
}

#define adc100m14b4cha_lib_cfg_test18_desc "It checks the threshold trigger hysteresis read/write"
static void adc100m14b4cha_lib_cfg_test18(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg;
	uint32_t value;
	int i;

	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_THR;
	for (i = 0; i < 4; ++i) {
		cfg.route_to = i;
		cfg.flags = 0;
		value = random() & 0xFFFF;
		adc100m14b4cha_lib_cfg_test_readback(adc, &cfg,
						     ADC_CONF_TRG_THR_HYSTERESIS, value);
	}
}

#define adc100m14b4cha_lib_cfg_test19_desc "It checks a full acquisition configuration read/write"
static void adc100m14b4cha_lib_cfg_test19(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg, cfg_rb, cfg_orig;
	uint32_t value;
	int err;

	/* store original */
	memset(&cfg_orig, 0, sizeof(struct adc_conf));
	cfg_orig.type = ADC_CONF_TYPE_ACQ;
	adc_set_conf_mask(&cfg_orig, ADC_CONF_ACQ_PRE_SAMP);
	adc_set_conf_mask(&cfg_orig, ADC_CONF_ACQ_POST_SAMP);
	adc_set_conf_mask(&cfg_orig, ADC_CONF_ACQ_UNDERSAMPLE);
	adc_set_conf_mask(&cfg_orig, ADC_CONF_ACQ_N_SHOTS);
	err = adc_retrieve_config(adc, &cfg_orig);
	m_assert_int_eq(0, err);

	/* write random config */
	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_ACQ;
	value = random();
	adc_set_conf(&cfg, ADC_CONF_ACQ_PRE_SAMP, value);
	value = random() + 2;
	adc_set_conf(&cfg, ADC_CONF_ACQ_POST_SAMP, value);
	value = random() & 0xFF;
	adc_set_conf(&cfg, ADC_CONF_ACQ_UNDERSAMPLE, value);
	value = (random() & 0xF) +1;
	adc_set_conf(&cfg, ADC_CONF_ACQ_N_SHOTS, value);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);

	/* validate random config */
	memcpy(&cfg_rb, &cfg_orig, sizeof(struct adc_conf));
	err = adc_retrieve_config(adc, &cfg_rb);
	m_assert_int_eq(0, err);
	m_assert_int_eq(cfg.mask, cfg_rb.mask);
	m_assert_int_eq(cfg.value[ADC_CONF_ACQ_PRE_SAMP],
			cfg_rb.value[ADC_CONF_ACQ_PRE_SAMP]);
	m_assert_int_eq(cfg.value[ADC_CONF_ACQ_POST_SAMP],
			cfg_rb.value[ADC_CONF_ACQ_POST_SAMP]);
	m_assert_int_eq(cfg.value[ADC_CONF_ACQ_UNDERSAMPLE],
			cfg_rb.value[ADC_CONF_ACQ_UNDERSAMPLE]);
	m_assert_int_eq(cfg.value[ADC_CONF_ACQ_N_SHOTS],
			cfg_rb.value[ADC_CONF_ACQ_N_SHOTS]);
}

#define adc100m14b4cha_lib_cfg_test20_desc "It checks a full channel configuration read/write"
static void adc100m14b4cha_lib_cfg_test20(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg, cfg_rb, cfg_orig;
	uint32_t value;
	int err, i;

	/* store original */
	for (i = 0; i < 4; ++i) {
		memset(&cfg_orig, 0, sizeof(struct adc_conf));
		cfg_orig.type = ADC_CONF_TYPE_CHN;
		cfg_orig.route_to = i;

		adc_set_conf_mask(&cfg_orig, ADC_CONF_CHN_RANGE);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_CHN_TERMINATION);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_CHN_OFFSET);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_CHN_SATURATION);
		err = adc_retrieve_config(adc, &cfg_orig);
		m_assert_int_eq(0, err);

		/* write random config */
		memset(&cfg, 0, sizeof(struct adc_conf));
		cfg.type = ADC_CONF_TYPE_CHN;
		cfg.route_to = i;

		value = 0x11;
		adc_set_conf(&cfg, ADC_CONF_CHN_RANGE, value);
		value = random() & 0x1;
		adc_set_conf(&cfg, ADC_CONF_CHN_TERMINATION, value);
		value = random() & 0xFFF;
		adc_set_conf(&cfg, ADC_CONF_CHN_OFFSET, value);
		value = random() & 0x7FFF;
		adc_set_conf(&cfg, ADC_CONF_CHN_SATURATION, value);
		err = adc_retrieve_config(adc, &cfg);
		m_assert_int_eq(0, err);

		/* validate random config */
		memcpy(&cfg_rb, &cfg_orig, sizeof(struct adc_conf));
		err = adc_retrieve_config(adc, &cfg_rb);
		m_assert_int_eq(0, err);
		m_assert_int_eq(cfg.mask, cfg_rb.mask);
		m_assert_int_eq(cfg.value[ADC_CONF_CHN_RANGE],
				cfg_rb.value[ADC_CONF_CHN_RANGE]);
		m_assert_int_eq(cfg.value[ADC_CONF_CHN_TERMINATION],
				cfg_rb.value[ADC_CONF_CHN_TERMINATION]);
		m_assert_int_eq(cfg.value[ADC_CONF_CHN_OFFSET],
				cfg_rb.value[ADC_CONF_CHN_OFFSET]);
		m_assert_int_eq(cfg.value[ADC_CONF_CHN_SATURATION],
				cfg_rb.value[ADC_CONF_CHN_SATURATION]);
	}
}

#define adc100m14b4cha_lib_cfg_test21_desc "It checks a full external trigger configuration read/write"
static void adc100m14b4cha_lib_cfg_test21(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg, cfg_rb, cfg_orig;
	uint32_t value;
	int err;

	/* store original */
	memset(&cfg_orig, 0, sizeof(struct adc_conf));
	cfg_orig.type = ADC_CONF_TYPE_TRG_EXT;
	adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_EXT_ENABLE);
	adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_EXT_POLARITY);
	adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_EXT_DELAY);
	err = adc_retrieve_config(adc, &cfg_orig);
	m_assert_int_eq(0, err);

	/* write random config */
	memset(&cfg, 0, sizeof(struct adc_conf));
	cfg.type = ADC_CONF_TYPE_TRG_EXT;
	value = random() & 0x1;
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_ENABLE, value);
	value = random() & 0x1;
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_POLARITY, value);
	value = random();
	adc_set_conf(&cfg, ADC_CONF_TRG_EXT_DELAY, value);
	err = adc_retrieve_config(adc, &cfg);
	m_assert_int_eq(0, err);

	/* validate random config */
	memcpy(&cfg_rb, &cfg_orig, sizeof(struct adc_conf));
	err = adc_retrieve_config(adc, &cfg_rb);
	m_assert_int_eq(0, err);
	m_assert_int_eq(cfg.mask, cfg_rb.mask);
	m_assert_int_eq(cfg.value[ADC_CONF_TRG_EXT_ENABLE],
			cfg_rb.value[ADC_CONF_TRG_EXT_ENABLE]);
	m_assert_int_eq(cfg.value[ADC_CONF_TRG_EXT_POLARITY],
			cfg_rb.value[ADC_CONF_TRG_EXT_POLARITY]);
	m_assert_int_eq(cfg.value[ADC_CONF_TRG_EXT_DELAY],
			cfg_rb.value[ADC_CONF_TRG_EXT_DELAY]);
}

#define adc100m14b4cha_lib_cfg_test22_desc "It checks a full threshold trigger configuration read/write"
static void adc100m14b4cha_lib_cfg_test22(struct m_test *m_test)
{
	struct adc_dev *adc = m_test->suite->private;
	struct adc_conf cfg, cfg_rb, cfg_orig;
	uint32_t value;
	int err, i;

	/* store original */
	for (i = 0; i < 4; ++i) {
		memset(&cfg_orig, 0, sizeof(struct adc_conf));
		cfg_orig.type = ADC_CONF_TYPE_TRG_THR;
		cfg_orig.route_to = i;

		adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_THR_ENABLE);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_THR_POLARITY);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_THR_DELAY);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_THR_THRESHOLD);
		adc_set_conf_mask(&cfg_orig, ADC_CONF_TRG_THR_HYSTERESIS);
		err = adc_retrieve_config(adc, &cfg_orig);
		m_assert_int_eq(0, err);

		/* write random config */
		memset(&cfg, 0, sizeof(struct adc_conf));
		cfg.type = ADC_CONF_TYPE_TRG_THR;
		cfg.route_to = i;

		value = random() & 0x1;
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_ENABLE, value);
		value = random() & 0x1;
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_POLARITY, value);
		value = random();
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_DELAY, value);
		value = random() & 0xFFFF;
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_THRESHOLD, value);
		value = random() & 0xFFFF;
		adc_set_conf(&cfg, ADC_CONF_TRG_THR_HYSTERESIS, value);
		err = adc_retrieve_config(adc, &cfg);
		m_assert_int_eq(0, err);

		/* validate random config */
		memcpy(&cfg_rb, &cfg_orig, sizeof(struct adc_conf));
		err = adc_retrieve_config(adc, &cfg_rb);
		m_assert_int_eq(0, err);
		m_assert_int_eq(cfg.mask, cfg_rb.mask);
		m_assert_int_eq(cfg.value[ADC_CONF_TRG_THR_ENABLE],
				cfg_rb.value[ADC_CONF_TRG_THR_ENABLE]);
		m_assert_int_eq(cfg.value[ADC_CONF_TRG_THR_POLARITY],
				cfg_rb.value[ADC_CONF_TRG_THR_POLARITY]);
		m_assert_int_eq(cfg.value[ADC_CONF_TRG_THR_DELAY],
				cfg_rb.value[ADC_CONF_TRG_THR_DELAY]);
		m_assert_int_eq(cfg.value[ADC_CONF_TRG_THR_THRESHOLD],
				cfg_rb.value[ADC_CONF_TRG_THR_THRESHOLD]);
		m_assert_int_eq(cfg.value[ADC_CONF_TRG_THR_HYSTERESIS],
				cfg_rb.value[ADC_CONF_TRG_THR_HYSTERESIS]);
	}
}


#define adc100m14b4cha_lib_cfg_m_test(_n) m_test_desc_loop(NULL, \
							   adc100m14b4cha_lib_cfg_test ## _n, \
							   NULL, \
							   adc100m14b4cha_lib_cfg_test ##_n ## _desc, \
							   LOOP)

static struct m_test adc100m14b4cha_lib_tests[] = {
	/* Acquisition parameters */
	adc100m14b4cha_lib_cfg_m_test(1),
	adc100m14b4cha_lib_cfg_m_test(2),
	adc100m14b4cha_lib_cfg_m_test(3),
	adc100m14b4cha_lib_cfg_m_test(4),
	adc100m14b4cha_lib_cfg_m_test(5),
	adc100m14b4cha_lib_cfg_m_test(6),
	/* Channel parameters */
	adc100m14b4cha_lib_cfg_m_test(7),
	adc100m14b4cha_lib_cfg_m_test(8),
	adc100m14b4cha_lib_cfg_m_test(9),
	adc100m14b4cha_lib_cfg_m_test(10),
	/* External trigger parameters */
	adc100m14b4cha_lib_cfg_m_test(11),
	adc100m14b4cha_lib_cfg_m_test(12),
	adc100m14b4cha_lib_cfg_m_test(13),
	/* Threshold trigger parameters */
	adc100m14b4cha_lib_cfg_m_test(14),
	adc100m14b4cha_lib_cfg_m_test(15),
	adc100m14b4cha_lib_cfg_m_test(16),
	adc100m14b4cha_lib_cfg_m_test(17),
	adc100m14b4cha_lib_cfg_m_test(18),
	/* Time trigger parameters */
	/* Acquisition parameters */
	adc100m14b4cha_lib_cfg_m_test(19),
	/* Channel parameters */
	adc100m14b4cha_lib_cfg_m_test(20),
	/* External trigger parameters */
	adc100m14b4cha_lib_cfg_m_test(21),
	/* Threshold trigger parameters */
	adc100m14b4cha_lib_cfg_m_test(22),
	/* time trigger parameters */
};
struct m_suite adc100m14b4cha_lib_cfg_suite = {
	.name = "FMC ADC 100M 4Bit 4Channel",
	.desc = "this collection of tests goes through all the configuration parameters and verify that we can read/write them correctly",
	.flags = M_ERRNO_CHECK | M_ERRNO_FUNC,
	.private = NULL,
	.tests = adc100m14b4cha_lib_tests,
	.test_count = M_ARRAY_SIZE(adc100m14b4cha_lib_tests),
	.set_up = adc100m14b4cha_lib_set_up,
	.tear_down = adc100m14b4cha_lib_tear_down,
	.strerror = NULL,
};
